<?php
/**
 * Builds a select box and automatically matches selection
 * <code>
 * echo build_select('some_id', 'area[name]', array(
 *   '1' => 'Monday',
 *   '2' => 'Tuesday',
 *   '3' => 'Wednesday'
 * ), '2'); // Will match Tuesday and have it selected
 * </code>
 * @param string $id
 * @param string $name
 * @param array $options Value/name pairs
 * @param string $selected The value to be initially selected
 * @return string
 */
function build_select($id, $name, $options, $selected = false, $multi = false, $parameters = false) {
	$params = array(
		'name' => $name,
		'id' => $id
	);
	if($multi !== false) $params['multiple'] = 'multiple';
	
	$params = empty($parameters) ? $params : array_merge($parameters, $params);
	
	$param_string = '';
	foreach($params as $k => $v) $params_string.= "{$k}=\"{$v}\" ";
	
	$str = "<select ".trim($params_string).">";
	if(empty($options)) $options = array();
	foreach($options as $value => $string) {
		if($selected == $value) {
			$str.= "<option value=\"{$value}\" selected=\"selected\">{$string}</option>\n";
		} else {
			$str.= "<option value=\"{$value}\">{$string}</option>\n";
		}
	}
	$str.= "</select>";
	return $str;
}

/**
 * Builds a menu (ul, li and a) and automatically matches selection
 * <code>
 * echo build_menu('some_id', 'some_class', array(
 *   'Click me' => 'index.php',
 *   'Another link' => 'page2.php',
 *   'Different link' => 'page3.php'
 * ), 'index.php'); // Will match 'Click me' and apply the 'active' class
 * </code>
 * @param string $id
 * @param string $class_name
 * @param array $options Value/name pairs
 * @param string $selected The value to be initially selected
 * @return string
 */
function build_menu($id, $class_name, $options, $selected = false) {
	$str = "<ul class=\"{$class_name}\" id=\"{$id}\">";
	foreach($options as $string => $value) {
		$n = (substr($value, 0, 4) == 'http' ? 'target="_blank"' : '');
		if($selected && $selected == $value) {
			$str.= "<li class=\"active\"><a href=\"{$value}\" title=\"{$string}\">{$string}</a></li>\n";
		} else {
			$str.= "<li><a href=\"{$value}\" title=\"{$string}\" {$n}>{$string}</a></li>\n";
		}
	}
	$str.= "</ul>";
	return $str;
}

function currencyCodeToSymbol($code) {
	$symbol = '$';
	switch($code) {
		case 'USD':
			$symbol = '$'; break;
		case 'GBP':
			$symbol = '&pound;'; break;
		case 'EUR':
			$symbol = '&euro;'; break;
		case 'YEN':
			$symbol = '&yen;'; break;
		case 'F':
			$symbol = '&#8355;'; break;
		case 'L':
			$symbol = '&#8356;'; break;
	}
	
	return $symbol;
}

function hex2rgb($h, $a=false) {
	if ($h[0] == '#') $h = substr($h,1);
	
	if (strlen($h) == 3) {
	  $h = str_split($h,1);
	  foreach ($h as &$c) $c .= $c;
	} elseif (strlen($h) == 6) {
	  $h = str_split($h,2);
	} else return null;
	
	foreach ($h as &$t) {
	  $t = base_convert($t,16,10);
	}
	return $a ? array_combine(array('red','green','blue'),$h) : implode(',',$h);
}

function rgb2hex($r, $g, $b, $uppercase=false, $shorten=false) {
	// The output
	$out = "";
	
	// If shorten should be attempted, determine if it is even possible
	if ($shorten && ($r + $g + $b) % 17 !== 0) $shorten = false;
	
	// Red, green and blue as color
	foreach (array($r, $g, $b) as $c) {
		// The HEX equivalent
		$hex = base_convert($c, 10, 16);
		
		// If it should be shortened, and if it is possible, then
		// only grab the first HEX character
		if ($shorten) $out .= $hex[0];
		
		// Otherwise add the full HEX value (if the decimal color
		// is below 16 then we have to prepend a 0 to it)
		else $out .= ($c < 16) ? ("0".$hex) : $hex;
	}
	// Package and away we go!
	return $uppercase ? strtoupper($out) : $out;
}

function generateGradient($startColorStr, $endColorStr, $percentFade) {
	if(empty($endColorStr)) $endColorStr = 'ffffff';
	
	$startColor = hex2rgb($startColorStr, true);
	$endColor = hex2rgb($endColorStr, true);
	
	$diffRed = $endColor['red'] - $startColor['red'];
	$diffGreen = $endColor['green'] - $startColor['green'];
	$diffBlue = $endColor['blue'] - $startColor['blue'];
	
	$diffRed = ceil(($diffRed * $percentFade) + $startColor['red']);
	$diffGreen = ceil(($diffGreen * $percentFade) + $startColor['green']);
	$diffBlue = ceil(($diffBlue * $percentFade) + $startColor['blue']);
	
 return rgb2hex($diffRed, $diffGreen, $diffBlue);
}
?>