<?php
// Undo magic_quotes
if(get_magic_quotes_gpc()) {
	$in = array(&$_GET, &$_POST, &$_COOKIE);
	while(list($k, $v) = each($in)) {
		foreach($v as $key => $val) {
			if(!is_array($val)) {
				$in[$k][$key] = stripslashes($val);
				continue;
			}
			$in[] =& $in[$k][$key];
		}
	}
	unset($in);
}

if(function_exists('date_default_timezone_set')) {
	date_default_timezone_set('UTC');
}

/**
 * Objects are preferred to arrays for storage of information
 * This class provides methods and such to all Objects
 * Can also be subclassed to extend data accessing functionalities
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.1
 * @package Eternity
 */
class Object {

	/**
	 * Merges all arguments into one Object
	 * <code>
	 * $obj = new Object($oneArray, $otherArray);
	 * </code>
	 * @param array|object|Object,...
	 * @return Object
	 */
	 
	public function __construct() {
		$arguments = func_get_args();
		foreach($arguments as $idx => $argument) {
			if(is_array($argument) || is_object($argument)) {
				foreach($argument as $key => $value) {
					if(is_array($value)) $value = new Object($value);
					$this->{$key} = $value;
				}
			} else {
				$this->{$idx} = $argument;
			}
		}
	}
	
	public function attach($key, $value) {
		if(!is_object($this->attachments)) $this->attachments = new stdClass();
		$this->attachments->{$key} = $value;
		return $this;
	}
	
	/**
	 * Returns true if all the passed keys are set
	 * <code>
	 * if($obj->keysExist('key', 'key2')) doSomething();
	 * </code>
	 * @param string... $key The keys to check
	 * @return bool
	 */
	
	public function keysExist() {
		$keys = func_get_args();
		foreach($keys as $k) {
			if(!isset($this->{$k})) return false;
		}
		return true;
	}
	
	public function push($key, $value = false) {
		$this->{$key} = splat($this->{$key});
		array_push($this->{$key}, $value);
		return $this;
	}
	
	public function asArray() {
		$array = array();
		if(!empty($this)) {
			foreach($this as $key => $value) {
				if($value instanceof Object) $value = $value->asArray();
				$array[$key] = $value;
			}
		}
		return $array;
	}
	
	public function __toString() {
		return json_encode($this);
	}
}

/**
 * Provides sorting arrays of objects based on their key values
 * @see http://uk2.php.net/manual/en/function.usort.php#87884
 * @author <jalil@measat.org>
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.1
 * @package Eternity
 */
class ObjectSorter {
	var $property;
	var $default;
	var $data;
	
	/**
	 * Sorts an array of objects
	 * @param array
	 * @param array $properties Index 0 to be the main property; 1 is the default (which must exist)
	 * @return $this
	 */
	 
	public function __construct($objects_array, $properties) {
		if(sizeof($properties) == 1) $properties[1] = $properties[0];
		$property = $properties[0];
		$default = $properties[1];
		$this->property = $property;
		$this->default = $default;
		usort($objects_array, array($this, '_compare'));
		$this->data = ($objects_array);
	}

	function _compare($apple, $orange) {
		$aprop = (isset($apple->{$this->property})) ? $this->property : $this->default;
		$bprop = (isset($orange->{$this->property})) ? $this->property : $this->default;
		if($apple->{$aprop} == $orange->{$bprop}) return 0;
		return ($apple->{$aprop} < $orange->{$bprop}) ? -1 : 1;
	}
}

/**
 * Extend this class to add 'event-driven' functionality
 * <code>
 * class MyClass extends Events {
 *     public function initialise() {
 *         //
 *     }
 * }
 * </code>
 * @see Base
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.1
 * @package Eternity
 */

class Events {
	/**
	 * Attaches a function to the specified stack
	 * Functions can be of 4 different types
	 * <b>Potential functions</b>
	 * - Functions
	 * - Class methods
	 * - Lambda functions
	 * - Pseudo-lambda functions
	 * <b>Functions</b>
	 * <code>
	 * function testFunction($argument) {
	 * 		echo $argument;
	 * }
	 * $someClass->addEvent('onStart', 'testFunction');
	 * </code>
	 * <b>Class methods</b>
	 * <code>
	 * $this->addEvent('onStart', array($this, 'classMethod'));
	 * </code>
	 * <b>Lambda functions</b>
	 * <code>
	 * $someClass->addEvent('onStart', func('
	 * 		echo $argument;
	 * '));
	 * </code>
	 * <b>Pseudo-lambda functions</b>
	 * <code>
	 * $someClass->addEvent('onStart', anonymous('
	 * 		echo "%1$s";
	 * '));
	 * </code>
	 * @param string $event Name of the event which will be later fired
	 * @param string|array $function The attached function
	 * @return object Current class
	 */
	 
	public function addEvent($event, $function) {
		$stack = &$this->internals->events;
		if(!isset($stack)) $stack = array();
		if(!isset($stack[$event])) $stack[$event] = array();
		$thisStack = array();
		$thisStack['event'] = $function;
		$stack[$event][] = $thisStack;
		return $this;
	}
	
	public function addEvents($stack) {
		if(empty($stack)) return false;
		foreach($stack as $event => $function) {
			$this->addEvent($event, $function);
		}
		return $this;
	}
	
	/**
	 * Fires a registered event
	 * If you are using a customised lambda function, arrays or objects passed as arguments <b>will be serialised</b>
	 * The first argument to ALL types of callbacks is an Event object, containing information about this event
	 * <code>
	 * $this->fireEvent('onStart', array('Some argument', 'Another argument'));
	 * $this->fireEvent('onSomethingElse', 'test'); // This also works
	 * </code>
	 * @param string $event The event to be fired
	 * @param array $args Array of arguments to be passed to the callback
	 */
	 
	public function fireEvent($event, $args = array()) {
		$stack = &$this->internals->events;
		if(!isset($stack[$event]) || empty($stack[$event])) return false;
		$args = splat($args); // Ensure it's an array
		// Create the Event
		$eventData = array(
			'type' => $event,
			'arguments' => $args,
			'callingClass' => &$this,
			'timeFired' => time()
		);
		$eventData['backtrace'] = debug_backtrace();
		$eventData['root'] = $eventData['backtrace'][sizeof($eventData['backtrace']) - 1]; // Where did this event get fired from?
		$theEvent = new Event($eventData);
		array_unshift($args, $theEvent);
		foreach($stack[$event] as $ev) {
			$func = $ev['event'];
			if(is_callable($func)) {
				call_user_func_array($func, $args);
			} elseif(is_string($func)) { // Custom eval solution for a JS-like lambda
				$this->anonymousEval($func, $args, $bind);
			}
		}
	}
	
	// Remove the whole stack for this type
	public function removeEvents($event) {
		$stack = &$this->internals->events;
		unset($stack[$event]);
		return $this;
	}
	
	// Remove the $nth event of type $event
	public function removeEventAtN($event, $n = 0) {
		$stack = &$this->internals->events;
		if($stack[$event][$n]) unset($stack[$event][$n]);
		return $this;
	}
	 
	protected function anonymousEval($code, $args) {
		if(!is_array($args)) {
			$new = array();
			$new[] = $args;
			unset($args);
			$args = $new;
		}
		if(!empty($args)) {
			foreach($args as &$arg) {
				if($arg === true) $arg = 'true';
				elseif($arg === false) $arg = 'false';
				elseif(is_array($arg) || is_object($arg)) $arg = serialize($arg);
			}
		}
		eval(vsprintf($code, $args));
	}
}

/**
 * Wrapper for create_function
 * @see Events
 * @see http://php.net/create_function
 */
function func($args, $code) {
	return create_function($args, $code);
}

/**
 * Wrapper for JS-like anonymous functions within the Events class
 * Only used for consistency - not technically required
 * @see Events
 */
function anonymous($code) {
	return $code;
}

/**
 * An instance will be passed as the first argument to every attached event
 * It contains useful information about the event
 * <b>Properties</b>
 * * type - The event type
 * * callingClass - Reference to the class which fired this event
 * * timeFired - The time this event was fired
 * * arguments - An array of all the arguments passed to the callback (Except this Event class)
 * @see Events
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.1
 * @package Eternity
 */

class Event {
	public function __construct($data) {
		foreach($data as $key => $value) $this->{$key} = $value;
	}
	
	public function getType() {
		return $this->type;
	}
	
	public function getCaller() {
		return $this->callingClass;
	}
}

/**
 * Almost all classes will want to extend Base to get functionality
 * such as storage, a Registry entry, event support
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.1
 * @package Eternity
 */
 
class Base extends Events {
	/**
	 * Sets $this->internals->className, provides $this->internals->storage, adds this instance to the Registry
	 * Also adds running $this->initialise and $this->shutdown for Moo-style classes
	 * <code>
	 * class Something extends Base {
	 *     public function initialise() {
	 *         //
	 *		 }
	 * }
	 * $mySomething = new Something(); // Will run initialise() - all arguments will be passed to it as well
	 * </code>
	 * @param mixed $x ... All arguments will be passed to $this->initialise() if defined
	 */

	public function __construct() {
		$this->internals = new Object();
		$this->internals->storage = new Object();
		$this->internals->events = array();
		$reverse = new ReflectionClass($this);
		$this->internals->className = $reverse->getName();
		$this->internals->classType = 'library';
		
		Registry::set($this->internals->className, $this); // Register this object
		
		$arguments = func_get_args();
		if(method_exists($this, 'initialise')) call_user_func_array(array($this, 'initialise'), $arguments);
		return $this;
	}
	
	/**
	 * Calls $this->shutdown and removes from the Registry
	 */
	
	public function __destruct() {
		if(class_exists('Registry') && Registry::fetch() instanceof Registry) Registry::remove($this->internals->className);
		$arguments = func_get_args();
		if(method_exists($this, 'shutdown')) call_user_func_array(array($this, 'shutdown'), $arguments);
	}
	
	/**
	 * Merges two hashes together, similar to JavaScript object merging
	 * <code>
	 * $this->setOptions($options);
	 * </code>
	 * @param string|object|Object $options Default options will be overridden by those present in here
	 * @param string|object|Object $defaults The default options
	 * @return mixed
	 */
	
	public function setOptions($options = false, $defaults = false) {
		if(!$defaults && isset($this->defaults)) $defaults = $this->defaults;
		$options = decode($options);
		$defaults = decode($defaults);
		$this->options = merge($options, $defaults);
		return $this;
	}
	
	public function setSettings($table, $databaseHandler = false) {
		if($databaseHandler === false) $databaseHandler = Registry::retrieve('database');
		if(!$databaseHandler instanceof Database) return false;
		$settings = $databaseHandler->getArray("SELECT `name`, `value` FROM {$table}");
		$this->settings = array();
		if(!empty($settings)) {
			foreach($settings as $i) {
				$this->settings[$i->name] = unescape($i->value, true);
				if(!is_numeric($this->settings[$i->name]) && $decode = decode($this->settings[$i->name])) {
					$this->settings[$i->name] = $decode;
					if(!is_array($this->settings[$i->name]) || is_object($this->settings[$i->name])) {
						$this->settings[$i->name] = unescape($this->settings[$i->name], true);
					}
				}
			}
		}
		$this->settings = new Object($this->settings);
		return $this;
	}
	
	/**
	 * Saves a value in the class storage
	 * Useful for caching results for later use
	 * @param string $key
	 * @param mixed $value
	 * @return The $value
	 */
	 
	public function store($key, &$value) {
		$storage = &$this->internals->storage;
		$storage->{$key} = $value;
		return $value;
	}
	
	/**
	 * Return a key from the class storage
	 * @param string $key
	 * @param mixed $default If the key isn't found, return this instead
	 */
	
	public function retrieve($key, $default = false) {
		$storage = &$this->internals->storage;
		if(isset($storage->{$key})) return $storage->{$key};
		$storage->{$key} = $default;
		return $storage->{$key};
	}
	
	public function deleteMySQLObject($table, $id, $field = 'id') {
		$table = escape($table);
		$id = escape($id, true);
		query("DELETE FROM {$this->options->prefix}{$table} WHERE `{$field}` = '{$id}'");
		return $this;
	}
	
	public function insertMySQLObject($table, $object) {
		$table = escape($table);
		if(empty($object)) return false;
		$object = new Object($object);
		$keys = array();
		$values = array();
		$id = $object->id;
		foreach($object as $key => $value) {
			if($key == 'data') {
				if(!empty($id)) {
					foreach($value as $dkey => $dvalue) {
						$this->insertDataKey("data[{$dkey}]", $dvalue, $id, $table);
					}
				}
			} else {
				$keys[] = "`{$key}`";
				if(is_object($value) || is_array($value)) $values[] = "'".escape(encode($value))."'";
				elseif(is_numeric($value)) $values[] = escape($value);
				else $values[] = "'".escape($value)."'";
			}
		}
		$keys = implode(',', $keys);
		$values = implode(',', $values);
		$success = query("INSERT INTO {$this->options->prefix}{$table} ({$keys}) VALUES ({$values})");
		return $success;
	}
	
	public function getMySQLObjectGroup($table, $parent = false, $match_column = 'id', $order_by = 'id', $order = 'desc', $getData = true) {
	  $identifier = escape(trim($identifier));
	  $match_column = escape($match_column);
	  $table = escape($table);
	 if(!empty($parent) && !empty($parent['value'])) {
			if(empty($parent['column'])) $parent['column'] = 'id';
			$parent['table'] = plural($parent['table']);
			$order_by = 'a.'.$order_by;
			$objs = getArray("SELECT * FROM {$this->options->prefix}{$table} a, {$this->options->prefix}{$parent['table']} b WHERE b.{$parent['column']} = '{$parent['value']}' && b.{$parent['match_column']} = a.{$match_column} ORDER BY {$order_by} {$order}");
		} elseif(!empty($parent) && $parent['value'] == 0) {
			if(empty($parent['column'])) $parent['column'] = 'id';
			$parent['table'] = plural($parent['table']);
			$order_all_by = $order_by;
			$order_by = 'a.'.$order_by;
			
			if($parent['table'] == $table) {
				$objs_with_parents = getArray("SELECT * FROM {$this->options->prefix}{$table} a WHERE a.{$parent['column']} != 0 && a.{$parent['column']} IS NOT NULL ORDER BY {$order_by} {$order}");
			} else {
				$objs_with_parents = getArray("SELECT * FROM {$this->options->prefix}{$table} a, {$this->options->prefix}{$parent['table']} b WHERE b.{$parent['match_column']} = a.{$match_column} ORDER BY {$order_by} {$order}");
			}
			
			$all_objs = getArray("SELECT * FROM {$this->options->prefix}{$table} ORDER BY {$order_all_by} {$order}");
			
			$objs = array();
			
			if(!empty($all_objs) && !empty($objs_with_parents)) {
				if($parent['table'] == $table) {
					$remove_these = flattenByKey($objs_with_parents, $parent['match_column']);
					foreach($all_objs as $idx => $el) {
						if(!in_array($el->{$parent['match_column']}, $remove_these)) {
							$objs[] = $el;
						}
					}
				} else {
					$remove_these = flattenByKey($objs_with_parents, $parent['match_column']);
					foreach($all_objs as $idx => $el) {
						if(!in_array($el->{$match_column}, $remove_these)) {
							$objs[] = $el;
						}
					}
				}
			} else {
				$objs = $all_objs;
			}
	  } else {
			$objs = getArray("SELECT * FROM {$this->options->prefix}{$table} ORDER BY `{$order_by}` {$order}");
	  }
	  if($objs) {
			if($getData == true) {
				$table_s = singular($table);
				foreach($objs as &$obj) {
					$data = getArray("SELECT `key`, `value` FROM {$this->options->prefix}{$table}_data WHERE `{$table_s}_{$match_column}` = '{$obj->{$match_column}}'");
					if(!empty($data)) {
						$vars = array();
						foreach($data as $d) {
							$vars[$d->key] = $d->value;
						}
						$ssn = ssn_decode($vars);
						$obj = merge($obj, $ssn);
					//parseData($vars, $obj);
					}
				}
				/*
				$table_s = singular($table);
				$match_array = array();
				$data = array();
				$single_key = "{$table_s}_{$match_column}";
				foreach($objs as $obj) array_push($match_array, "'{$obj->{$match_column}}'");
				$data_raw = getArray("SELECT `{$single_key}`, `key`, `value` FROM {$this->options->prefix}{$table}_data WHERE `{$table_s}_{$match_column}` IN(".implode(',', $match_array).")");
				if(!empty($data_raw)) {
					foreach($data_raw as $d) {
						$data[$d->{$single_key}][$d->key] = $d->value;
					}
					foreach($objs as &$obj) {
						$ssn = ssn_decode($data[$obj->{$single_key}]);
						$obj = merge($obj, $ssn);
					}
				}
				*/
			}
			$this->fireEvent('onGetDatabaseObject', array($obj, $identifier, $match_column, $table, $getData)); // onGetDatabaseObject for any object fetched
			$table_cc = plural($table);
			$event = camelcase("onGetDatabase {$table_cc}");
			$this->fireEvent($event, array($objs, $identifier, $match_column, $table, $getData)); // onGetDatabaseCampaigns
	  }
	  return $objs;
	}
	
	public function getMySQLObject($identifier, $table, $match_column = 'id', $getData = true) {
	  $identifier = escape(trim($identifier));
	  $match_column = escape($match_column);
	  $table = escape(plural($table));
	  $obj = getRow("SELECT * FROM {$this->options->prefix}{$table} WHERE `{$match_column}` = '{$identifier}'");
	  if($obj) {
			if($getData == true) {
				$table_s = singular($table);
				$data = getArray("SELECT `key`, `value` FROM {$this->options->prefix}{$table}_data WHERE `{$table_s}_{$match_column}` = '{$identifier}'");
				if(!empty($data)) {
					$vars = array();
					foreach($data as $d) {
						$vars[$d->key] = $d->value;
					}
					$ssn = ssn_decode($vars);
					$obj = merge($obj, $ssn);
				//parseData($vars, $obj);
				}
			}
			$this->fireEvent('onGetDatabaseObject', array($obj, $identifier, $match_column, $table, $getData)); // onGetDatabaseObject for any object fetched
			$table_cc = singular($table);
			$event = camelcase("onGetDatabase {$table_cc}");
			$this->fireEvent($event, array($obj, $identifier, $match_column, $table, $getData)); // onGetDatabaseCampaign
	  }
	  return $obj;
	}
	
	public function deleteDataKey($key, $id, $type, $deleteArrayData = false) {
		$id = escape($id);
		$type = singular($type);
		query("DELETE FROM {$this->options->prefix}{$type}s_data WHERE `{$type}_id` = '{$id}' && `key` = '{$key}'");
		if($deleteArrayData == true) { // Also remove stuff like event[whatever] etc etc as well as the fixed key
			query("DELETE FROM {$this->options->prefix}{$type}s_data WHERE `{$type}_id` = '{$id}' && `key` LIKE '{$key}[%'");
		}
		return $this;
	}
	
	public function insertDataKey($key, $value, $id, $type) {
		$id = escape($id);
		$type = singular($type);
		if(is_object($value) || is_array($value)) {
			$encoded_value = escape(encode($value));
		} else {
			$encoded_value = escape($value);
		}
		query("DELETE FROM {$this->options->prefix}{$type}s_data WHERE `{$type}_id` = '{$id}' && `key` = '{$key}'"); // Remove duplicates
		if($encoded_value !== '{}' && !$encoded_value !== '[]' && $encoded_value !== ''){//!empty($encoded_value)) {
		 query("INSERT INTO {$this->options->prefix}{$type}s_data (`{$type}_id`, `key`, `value`) VALUES ('{$id}', '{$key}', '{$encoded_value}')");
		}
		return $this;
	}
	
	public function getDataKey($key, $id, $type) {
		$id = escape($id);
		$type = singular($type);
		$key = strtolower(escape($key));
		$obj = new Object();
		if($key == '*') {
			$key_search = "";
		} else {
			$key_search = " && `key` = '{$key}'";
		}
		$data = getArray("SELECT `key`, `value` FROM {$this->options->prefix}{$type}s_data WHERE `{$type}_id` = '{$id}' {$key_search}");
		if(!empty($data)) {
		  $vars = array();
		  foreach($data as $d) {
				$vars[$d->key] = $d->value;
		  }
				$ssn = ssn_decode($vars);
				$obj = merge($obj, $ssn);
			//parseData($vars, $obj);
		}
		if($key == '*') {
			return $obj;
		}
		return $obj->{$key};
	}
}

/**
 * Extend this class for simple use of a singleton patern
 * This class cannot extend Base!
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.1
 * @package Eternity
 */

class Singleton {
	private static $_instance = null;
	
	private function __construct() {  }
	/**
	 * Return the current singleton
	 * <code>
	 * class Something extends Singleton {
	 *	public function shoot() {
	 *		$klass = self::fetch(); // $klass is now the singleton instance used in place of $this
	 *	}
	 * }
	 * Something::shoot(); // Shoot can do anything a 'normalton' can do!
	 * </code>
	 */
	 
	static public function fetch() {
	  if(@is_null(self::$_instance)) self::$_instance = new self();
	  return self::$_instance;
	}
}
?>