<?php
/**
 * Fires POST requests cross-domain
 * Can be used to simulate cross-domain AJAX, or postbacks
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.1
 * @package Eternity
 */
class Request extends Base {
	protected $defaults = array('timeout' => 10, 'method' => 'get', 'cache' => false, 'cacheDir' => 'Cache/', 'cacheLength' => 1800);
	
	public function trimCache() {
		if(!$this->options->cache) return $this;
		$files = getFiles($this->options->cacheDir);
		if(!empty($files)) {
			foreach($files as $file) {
				if(gmtime() - gmdate('U', filemtime($file)) >= $this->options->cacheLength) {
					@unlink($file);
				}
			}
		}
		return $this;
	}
	
	/**
	 * This initialises the request that you wish to forward
	 * The optional $options parameter can contain the arguments you wish to make use of in your send() calls
	 * <code>
	 * $request = new Request("{method:'post', url:'http://www.adobe.com'}");
	 * </code>
	 * @param string|Object $options Hash or object of options
	 */
	 
	public function initialise($options = false) {
		$this->setOptions($options);
		$this->trimCache();
		return $this;
	}
	
	/**
	 * Sends the request and returns the HTTP response, stripped of any headers
	 * If you pass options to this function, they will replace those set in initialise()	
	 * Note: You may also pass the $data as a query string
	 * <code>
	 * $request = new Request();
	 * $data = array(
	 *   'something' => 'value'
	 * );
	 * $httpResponse = $request->send("{url:'http://adobe.com'}", $data);
	 * </code>
	 * @param string|object $options Hash or object of options
	 * @param array|object $data Data to send as a get or post request
	 */
	 
	public function send() {
		$arguments = func_get_args();
		if(sizeof($arguments) == 2) { // We need to set the path options
			if($this->options == $this->defaults) $this->setOptions($arguments[0]);
			else $this->setOptions($arguments[0], $this->options);
			$data = $arguments[1];
		} else { // We're using $this->options
			$data = $arguments[0];
		}
		$this->options->method = strtoupper($this->options->method);
		if(is_array($data) || is_object($data)) $data = http_build_query($data);
		$parsed = @parse_url($this->options->url);
		if(!$parsed['path']) $parsed['path'] = '/';
		if($this->options->method == 'GET') $parsed['path'].="?{$data}";
		// Is this call in the cache?
		if($this->options->cache) {
			$hash = $this->getHash($data);
			if(file_exists($this->options->cacheDir.$hash)) {
				if(time() - filemtime($this->options->cacheDir.$hash) <= $this->options->cacheLength) {
					return file_get_contents($this->options->cacheDir.$hash);
				}
			}
		}
		// No - carry on :)
		$fp = @fsockopen($parsed['host'], '80');
		if($fp) {
			fputs($fp, "{$this->options->method} {$parsed['path']} HTTP/1.1\r\n");
			fputs($fp, "Host: {$parsed['host']}\r\n");
			fputs($fp, "Content-type: application/x-www-form-urlencoded\r\n");
			if($this->options->method == 'POST') fputs($fp, "Content-length: ".strlen($data)."\r\n");
			fputs($fp, "Connection: close\r\n\r\n");
			if($this->options->method == 'POST') fputs($fp, $data . "\r\n\r\n");
			while(!feof($fp)) $buf .= fgets($fp, 1024);
			fclose($fp);
		} else {
			return false;
		}
		if($d = strpos($buf, "\n\r")) $buf = trim(substr($buf, $d)); // Cut out the headers
		// Are we using the cache? Let's write to it
		if($this->options->cache) {
			@unlink($this->options->cacheDir.$hash);
			$file = new File($this->options->cacheDir.$hash, 'w');
			$file->write($buf);
		}
		return $buf;
	}
	
	protected function getHash($data) {
		return md5($this->options->url.$data);
	}
}

/**
 * Allows Requests to be sent via CURL
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.1
 * @package Eternity
 */
class Request_Curl extends Request {
	public function send() {
		$arguments = func_get_args();
		if(sizeof($arguments) == 2) { // We need to set the path options
			if($this->options == $this->defaults) $this->setOptions($arguments[0]);
			else $this->setOptions($arguments[0], $this->options);
			$data = $arguments[1];
		} else { // We're using $this->options
			$data = $arguments[0];
		}
		$this->options->method = strtoupper($this->options->method);
		if(is_array($data) || is_object($data)) $data = http_build_query($data);
		$parsed = @parse_url($this->options->url);
		if(!$parsed['path']) $parsed['path'] = '/';
		if($this->options->method == 'GET') $parsed['path'].="?{$data}";
		// Is this call in the cache?
		if($this->options->cache) {
			$hash = $this->getHash($data);
			if(file_exists($this->options->cacheDir.$hash)) {
				if(time() - filemtime($this->options->cacheDir.$hash) <= $this->options->cacheLength) {
					return file_get_contents($this->options->cacheDir.$hash);
				}
			}
		}
		// Curl
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $parsed['scheme'].'://'.$parsed['host'].(empty($parsed['port']) ? '' : ':'.$parsed['port']).$parsed['path']);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_TIMEOUT, $this->options->timeout);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		if($this->options->method == 'POST') {
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
		}
		
		if(ini_get('open_basedir') == '' && (ini_get('safe_mode') == 'Off' || ini_get('safe_mode') == '')) {
			curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
			$buf = curl_exec($ch);
		} else {
			$buf = $this->curl_redir_exec($ch);
		}
		
		if($buf === false) return false;
		curl_close($ch);
		
		// Are we using the cache? Let's write to it
		if($this->options->cache) {
			@unlink($this->options->cacheDir.$hash);
			$file = new File($this->options->cacheDir.$hash, 'w');
			$file->write($buf);
		}
		return $buf;
	}
	
	public function curl_redir_exec($ch) {
		static $curl_loops = 0;
		static $curl_max_loops = 20;
		 
		if($curl_loops++ >= $curl_max_loops) {
			$curl_loops = 0;
			return false;
		}
		 
		curl_setopt($ch, CURLOPT_HEADER, true);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		
		$data = curl_exec($ch);
		
		list($header, $data) = explode("\n\r", $data, 2);
		
		$http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		
		if($http_code == 301 || $http_code == 302) {
			$matches = array();
			preg_match('/Location:(.*?)\n/', $header, $matches);
			
			$url = @parse_url(trim(array_pop($matches)));
			
			if(!$url) { //couldn't process the url to redirect to
				$curl_loops = 0;
				return $data;
			}
			 
			$last_url = parse_url(curl_getinfo($ch, CURLINFO_EFFECTIVE_URL));
			 
			if(!$url['scheme']) $url['scheme'] = $last_url['scheme'];
			 
			if(!$url['host']) $url['host'] = $last_url['host'];
			 
			if(!$url['path']) $url['path'] = $last_url['path'];
			 
			$new_url = $url['scheme'] . '://' . $url['host'] . $url['path'] . ($url['query'] ? '?'.$url['query'] : '');
			 
			curl_setopt($ch, CURLOPT_URL, $new_url);
			
			$res = $this->curl_redir_exec($ch);
			if(!empty($res)) $res = trim($res);
			return $res;
		} else {
			$curl_loops = 0;
			return $data;
		}
	}
}
?>