<?php
/**
 * Server-side validating of values
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.2
 * @package Eternity
 */

class Validator extends Base {

	/**
	 * Accepts an array of values ($_POST?) to secure and an array of rules to validate against
	 * <code>
	 * $validator = new Validator($_POST, array(
	 *   'username' => array(
	 *     'minlength' => 3,
	 *     'maxlength' => 6
	 *    )
	 * ));
	 * if($validator->valid) {
	 *     // Everything passed
	 * } else {
	 *     // Do something with $validator->errors
	 * }
	 * </code>
	 * @param $data Array of values to be validated
	 * @param $rules Array of rules use
	 */
	
	public $data;
	public $valid = true;
	public $rules = array();
	public $errors = array();
	 
	public function initialise($data, $rules) {
		$this->data = $data;
		$this->rules = $rules;
		foreach($this->rules as $key => &$array) $array = new Object($array);
		foreach($this->data as $key => $value) {
			$this->errors[$key] = array();
			if(!$this->validate($key, $value, $this->rules[$key], $this->errors[$key])) $this->valid = false;
		}
	}
	
	protected function validate($key, $value, $rules, &$errorStack) {
		$length = strlen($value);
		if($rules->required) {
			if(!isset($value) || strlen($value) == 0) array_push($errorStack, "Is a required field.");
		}
		if($rules->same_as && $length > 0) {
			if($value != $this->data[$rules->same_as]) array_push($errorStack, "Must contain the same value as {$rules->same_as}");
		}
		if($rules->maxlength && $length > 0) {
			if(strlen($value) > $rules->maxlength) array_push($errorStack, "Must be less than {$rules->maxlength} characters long.");
		}
		if($rules->minlength && $length > 0) {
			if(strlen($value) < $rules->minlength) array_push($errorStack, "Must be more than {$rules->minlength} characters long.");
		}
		if($rules->length && $length > 0) {
			if(strlen($value) != $rules->length) array_push($errorStack, "Must be exactly {$rules->length} characters long.");
		}
		if($rules->alpha && $length > 0) {
			if(!preg_match('/^[a-zA-Z\s]+$/u', $value)) array_push($errorStack, "Can only contain alphabetic characters.");
		}
		if($rules->alphanum && $length > 0) {
			if(!preg_match('/^[a-zA-Z0-9\s]+$/u', $value)) array_push($errorStack, "Can only contain alphanumeric characters.");
		}
		if($rules->int && $length > 0) {
			if(!is_numeric($value)) array_push($errorStack, "Can only contain numeric characters.");
		}
		if($rules->englishdate && $length > 0) { // Checks dd/mm/yyyy
			$d = explode('/', $value);
			if(!$d || !checkDate((int)$d[1], (int)$d[0], (int)$d[2])) array_push($errorStack, "Must be a valid English date (dd/mm/yyyy).");
		}
		if($rules->simplestring && $length > 0) { // Allows alphanumeric and _ - for usernames and such
			if(!preg_match('/^[\w]+$/u', $value)) array_push($errorStack, "Can only contain alphanumeric characters, - and _.");
		}
		if($rules->email && $length > 0) {
			if(!preg_match('/^[^0-9][a-zA-Z0-9_\-\+]+([.][a-zA-Z0-9_\-\+]+)*[@][a-zA-Z0-9_\-]+([.][a-zA-Z0-9_\-]+)*[.][a-zA-Z]{2,4}$/', $value)) array_push($errorStack, "Must be a valid email address.");
		}
		if($rules->hex && $length > 0) {
			if(!preg_match('/^#(?:(?:[a-f\d]{3}){1,2})$/i', $value)) array_push($errorStack, "Must be a valid hex code (#000000).");
		}
		if($rules->url && $length > 0) {
			if(stripos($value, 'http://') === false) $value = 'http://'.$value;
			if(!preg_match("/^(((http(s?))|(ftp))\:\/\/)?(www.|[a-zA-Z].)[a-zA-Z0-9\-\.]+\.(com|edu|gov|mil|net|org|biz|info|name|museum|us|ca|cn|ly|uk|nz|me)(\:[0-9]+)*(\/($|[a-zA-Z0-9\.\,\;\?\'\\\+&%\$#\=~_\-]+))*$/", $value)) array_push($errorStack, "Must be a valid url.");
		}
		if(empty($errorStack)) return true;
		return false;
	}
}
?>